<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class DashboardFactory
{

    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    // requisição de retorno dos dados do dashboard Limpeza Finalizada
    public function getDadosLimpezaFinalizada($parametros)
    {

        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND,
                TO_CHAR(CA.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
                LCP.NMLOCALCLIENTE AS NMLOCALPAI, TO_CHAR(CA.DTULTATU, 'DD/MM/YYYY HH24:MI')DTULTATU, GCHK.NRGERENCIAMENTOCHK, GCHK.IDSITUACAOGERCHK
                      FROM FCM_CHECKLISTAGEND CA
                      JOIN FCM_LOCALCLI LC
                      ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                      JOIN FCM_LOCALCLI LCP
                      ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                      JOIN FCM_CHECKLIST CHK
                      ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                      JOIN OPERADOR F
                      ON F.CDOPERADOR = CA.CDOPERADOR
                      JOIN FCM_GERENCIAMENTOCHK GCHK
                      ON GCHK.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
                      WHERE CA.IDSITUACAOAGEND = '4'
                      AND GCHK.IDSITUACAOGERCHK = '1'
                      AND CA.CDOPERADOR IN ($parametros)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getDadosTarefaAtrasada($parametros)
    {

        $sql = "SELECT  NRCHECKLISTAGEND, NRLOCALCLIENTE, NRCHECKLIST, CDOPERADOR, IDSITUACAOAGEND, DTHORAAGEND, NMLOCALCLIENTE, LOCALPAI, NMLOCALPAI, HORAS AS DTHORAATRASO
               FROM ( SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND,
                      TO_CHAR(CA.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
                      LCP.NMLOCALCLIENTE AS NMLOCALPAI, TRUNC(SYSDATE - CA.DTHORAAGEND) * 24 + TRUNC(((SYSDATE - CA.DTHORAAGEND) - TRUNC (SYSDATE - CA.DTHORAAGEND)) * 24) AS HORAS,
                      TRUNC(((((SYSDATE - CA.DTHORAAGEND) - TRUNC(SYSDATE - CA.DTHORAAGEND)) * 24) - TRUNC(((SYSDATE - CA.DTHORAAGEND) - TRUNC(SYSDATE - CA.DTHORAAGEND)) *24))*60) AS MINUTOS, CA.CDOPERINCLUSAO
                            FROM FCM_CHECKLISTAGEND CA
                            JOIN FCM_LOCALCLI LC
                            ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                            JOIN FCM_LOCALCLI LCP
                            ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                            JOIN FCM_CHECKLIST CHK
                            ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                            JOIN OPERADOR F
                            ON F.CDOPERADOR = CA.CDOPERADOR
                            WHERE ((IDSITUACAOAGEND = '5') OR ((IDSITUACAOAGEND = '1') AND (CA.DTHORAAGEND <= SYSDATE)))) TAREFASATRASADAS
               WHERE CDOPERADOR IN ($parametros)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getDadosTarefaConcluida($parametros)
    {

        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND,
              TO_CHAR(CA.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTHORAAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
              LCP.NMLOCALCLIENTE AS NMLOCALPAI, TO_CHAR(CA.DTULTATU, 'DD/MM/YYYY HH24:MI') DTULTATU, GCHK.NRGERENCIAMENTOCHK, GCHK.IDSITUACAOGERCHK
                    FROM FCM_CHECKLISTAGEND CA
                    JOIN FCM_LOCALCLI LC
                    ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                    JOIN FCM_LOCALCLI LCP
                    ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                    JOIN FCM_CHECKLIST CHK
                    ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                    JOIN OPERADOR F
                    ON F.CDOPERADOR = CA.CDOPERADOR
                    JOIN FCM_GERENCIAMENTOCHK GCHK
                    ON GCHK.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
                    WHERE CA.IDSITUACAOAGEND = '4'
                    AND GCHK.IDSITUACAOGERCHK = '3'
                    AND TRUNC(CA.DTHORAAGEND) BETWEEN TRUNC(SYSDATE-30) AND TRUNC(SYSDATE+30)
                    AND CA.CDOPERADOR IN ($parametros)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function countTarefasConcluidas($parametros)
    {
        $sql = "SELECT COUNT(CA.NRCHECKLISTAGEND) AS QTDTAREFASCONCLUIDAS
                  FROM FCM_CHECKLISTAGEND CA
                  JOIN FCM_LOCALCLI LC
                  ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                  JOIN FCM_LOCALCLI LCP
                  ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                  JOIN FCM_CHECKLIST CHK
                  ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                  JOIN OPERADOR F
                  ON F.CDOPERADOR = CA.CDOPERADOR
                  JOIN FCM_GERENCIAMENTOCHK GCHK
                  ON GCHK.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
                  WHERE CA.IDSITUACAOAGEND = '4'
                  AND GCHK.IDSITUACAOGERCHK = '3'
                  AND TRUNC(SYSDATE) = TRUNC(CA.DTHORAAGEND)
                  AND CA.CDOPERADOR IN ({$parametros})";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function countTarefas($parametros)
    {
        $sql = "SELECT COUNT(CA.NRCHECKLISTAGEND) AS QTDTAREFAS
                  FROM FCM_CHECKLISTAGEND CA
                  JOIN FCM_LOCALCLI LC
                  ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                  JOIN FCM_LOCALCLI LCP
                  ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                  JOIN FCM_CHECKLIST CHK
                  ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                  JOIN OPERADOR F
                  ON F.CDOPERADOR = CA.CDOPERADOR
                  WHERE TRUNC(SYSDATE) = TRUNC(DTHORAAGEND)
                  AND CA.CDOPERADOR IN ($parametros)";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function findLocal()
    {
        $sql = "SELECT * FROM FCM_LOCALCLI";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getDadosUltimasTarefas($parametros)
    {
        $sql = "SELECT ULTTAR.CODIGO, ULTTAR.NRLOCALCLIENTE, ULTTAR.NRCHECKLIST, ULTTAR.CDOPERADOR, TO_CHAR(ULTTAR.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTULTATU, TO_CHAR(ULTTAR.DTULTATU, 'DD/MM/YYYY HH24:MI') HORARIO, 
        ULTTAR.NOME, ULTTAR.SITUACAO, ULTTAR.TIPO, FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI, LCP.NMLOCALCLIENTE AS NMLOCALPAI,
        ULTTAR.IDSITUACAOGERCHK as STATUS
        FROM (SELECT NRGERENCIAMENTOCHK AS CODIGO, NRLOCALCLIENTE, NRCHECKLIST,
                    CDOPERADOR,DTHORAAGEND, DTULTATU, 'Vistoria: '||NMGERENCIAMENTOCHK AS NOME , IDSITUACAOGERCHK AS SITUACAO,
                    'V' AS TIPO,
                    CASE IDSITUACAOGERCHK WHEN '1' THEN 'Aguardando vistoria'
                          WHEN '2' THEN 'Vistoria em andamento'
                          WHEN '3' THEN 'Tarefa Vistoriada'
                          ELSE NULL END AS IDSITUACAOGERCHK
                FROM FCM_GERENCIAMENTOCHK
                WHERE IDSITUACAOGERCHK = '3'

              UNION ALL

              SELECT NRCHECKLISTAGEND AS CODIGO, NRLOCALCLIENTE, NRCHECKLIST,
                      CDOPERADOR,DTHORAAGEND, DTULTATU, NMCHECKLISTAGEND AS NOME , IDSITUACAOAGEND AS SITUACAO,
                      'T' AS TIPO,
                      CASE IDSITUACAOAGEND WHEN '1' THEN 'Pendente'
                          WHEN '2' THEN 'Tarefa em andamento'
                          WHEN '3' THEN 'Tarefa Cancelada'
                          WHEN '4' THEN 'Tarefa Concluída'
                          WHEN '5' THEN 'Tarefa Atrasada'
                          ELSE NULL END AS IDSITUACAOAGEND
                FROM FCM_CHECKLISTAGEND
                WHERE IDSITUACAOAGEND = '4'
                ) ULTTAR
        JOIN OPERADOR FUNC
        ON ULTTAR.CDOPERADOR = FUNC.CDOPERADOR
        JOIN FCM_LOCALCLI LC
        ON LC.NRLOCALCLIENTE = ULTTAR.NRLOCALCLIENTE
        JOIN FCM_LOCALCLI LCP
        ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
        WHERE ULTTAR.CDOPERADOR IN ($parametros)
        ORDER BY DTULTATU DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getDadosResumoTarefas($parametros)
    {

        $sql = "SELECT ULTTAR.CODIGO, TO_CHAR(ULTTAR.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') DTULTATU, TO_CHAR(ULTTAR.DTULTATU, 'DD/MM/YYYY HH24:MI') HORARIO, ULTTAR.NOME, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI, LCP.NMLOCALCLIENTE AS NMLOCALPAI, 
        ULTTAR.IDSITUACAOGERCHK as STATUS, ULTTAR.DIAS, FUNC.NMOPERADOR, ULTTAR.NRLOCALCLIENTE, ULTTAR.CDOPERADOR
          FROM (SELECT NRGERENCIAMENTOCHK AS CODIGO, NRLOCALCLIENTE, NRCHECKLIST, CDOPERADOR, DTHORAAGEND, DTULTATU, 'Vistoria: ' || NMGERENCIAMENTOCHK AS NOME, IDSITUACAOGERCHK AS SITUACAO, 'V' AS TIPO,
                CASE IDSITUACAOGERCHK
                  WHEN '1' THEN 'Aguardando vistoria'
                  WHEN '2' THEN 'Vistoria em andamento'
                  WHEN '3' THEN 'Vistoria Realizada'
                  ELSE 'Tarefa Pendente' 
                END AS IDSITUACAOGERCHK, 
                  TRUNC(SYSDATE - DTULTATU) AS DIAS
                FROM FCM_GERENCIAMENTOCHK
                WHERE ((DTHORAAGEND = SYSDATE) OR ((DTHORAAGEND < SYSDATE)))
                
              UNION ALL
              
              SELECT NRCHECKLISTAGEND AS CODIGO, NRLOCALCLIENTE, NRCHECKLIST, CDOPERADOR, DTHORAAGEND, DTULTATU, NMCHECKLISTAGEND AS NOME, IDSITUACAOAGEND AS SITUACAO, 'T' AS TIPO,
                CASE IDSITUACAOAGEND
                  WHEN '1' THEN 'Tarefa Pendente'
                  WHEN '2' THEN 'Tarefa em andamento'
                  WHEN '3' THEN 'Tarefa Cancelada'
                  WHEN '4' THEN 'Tarefa Concluída'
                  WHEN '5' THEN 'Tarefa Atrasada'
                  ELSE NULL END AS IDSITUACAOAGEND,
                  TRUNC(SYSDATE - DTULTATU) AS DIAS
              FROM FCM_CHECKLISTAGEND
              WHERE ((DTHORAAGEND = SYSDATE) OR ((DTHORAAGEND < SYSDATE)))) ULTTAR
              
        JOIN OPERADOR FUNC 
          ON ULTTAR.CDOPERADOR = FUNC.CDOPERADOR
        JOIN FCM_LOCALCLI LC 
          ON LC.NRLOCALCLIENTE = ULTTAR.NRLOCALCLIENTE
        JOIN FCM_LOCALCLI LCP 
          ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
        WHERE ULTTAR.CDOPERADOR IN ({$parametros})
        ORDER BY DTULTATU DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getDadosChartTarefas($parametros)
    {
        $sql = "SELECT A.NMATIVIDADE, COUNT(CA.NRCHECKLISTAGEND) AS QUANTIDADE, A.DSCORIDATIVIDADE
                  FROM FCM_CHECKLISTAGEND CA
                  JOIN FCM_LOCALCLI LC
                  ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                  JOIN FCM_TIPOLOCALCLI TC
                  ON TC.NRTIPOLOCALCLI = LC.NRTIPOLOCALCLI
                  JOIN FCM_ATIVIDADECHK ACHK
                  ON ACHK.NRCHECKLIST = CA.NRCHECKLIST
                  JOIN FCM_ATIVIDADE A
                  ON A.NRATIVIDADE = ACHK.NRATIVIDADE
                  WHERE TRUNC(CA.DTHORAAGEND) BETWEEN TRUNC(SYSDATE-30) AND TRUNC(SYSDATE+30)
                  AND IDSITUACAOAGEND = '4'
                  AND TC.IDTIPOLOCALCLI = '3'
                  AND CA.CDOPERADOR IN ({$parametros})
                  GROUP BY A.NMATIVIDADE, A.DSCORIDATIVIDADE";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    // requisição para retorno dos dados para o grafico Não Conformidades Dashboard Supervisor
    public function getDadosChartNaoConformidades($parametros)
    {

        $sql = "SELECT A.NMATIVIDADE, A.DSCORIDATIVIDADE, COUNT(C.NRCRITERIOTARGERCHK) AS QUANTIDADE
                        FROM FCM_CRITERIOTARGERCHK C
                        JOIN FCM_TAREFAGERCHK TG
                          ON TG.NRTAREFAGERCHK = C.NRTAREFAGERCHK
                        JOIN FCM_GERENCIAMENTOCHK GCHK
                          ON GCHK.NRGERENCIAMENTOCHK = TG.NRGERENCIAMENTOCHK
                        JOIN FCM_LOCALCLI LC
                          ON LC.NRLOCALCLIENTE = GCHK.NRLOCALCLIENTE
                        JOIN FCM_TIPOLOCALCLI TC
                          ON TC.NRTIPOLOCALCLI = LC.NRTIPOLOCALCLI
                        JOIN FCM_ATIVIDADECHK ACHK
                          ON ACHK.NRCHECKLIST = GCHK.NRCHECKLIST
                        JOIN FCM_ATIVIDADE A
                          ON A.NRATIVIDADE = ACHK.NRATIVIDADE
                       WHERE IDMARCADO = 'S'
                         AND TC.IDTIPOLOCALCLI = '3'
                         AND TRUNC(GCHK.DTHORAAGEND) BETWEEN TRUNC(SYSDATE-30) AND TRUNC(SYSDATE+30)
                         AND TG.CDOPERADOR IN ({$parametros})
                    GROUP BY A.NMATIVIDADE, A.DSCORIDATIVIDADE
                    ORDER BY QUANTIDADE ASC";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function countTarefasConcluidasMensal($operadores)
    {

        $sql = "SELECT QUANTIDADE_CONCLUIDAS.NMATIVIDADE, QUANTIDADE_CONCLUIDAS.QUANTIDADE, QUANTIDADE_CONCLUIDAS.DSCORIDATIVIDADE,
                       ROUND((QUANTIDADE_CONCLUIDAS.QUANTIDADE * 100 / QUANTIDADE_TOTAL.QUANTIDADE_TOTAL),2 )AS PORCENTAGEM

                    FROM (SELECT AT.NMATIVIDADE, AT.NRATIVIDADE, AT.DSCORIDATIVIDADE,
                                 COUNT(CA.NRCHECKLISTAGEND) AS QUANTIDADE
                             FROM FCM_CHECKLISTAGEND CA
                             JOIN FCM_ATIVIDADECHK AC
                               ON AC.NRCHECKLIST = CA.NRCHECKLIST
                             JOIN FCM_ATIVIDADE AT
                               ON AT.NRATIVIDADE = AC.NRATIVIDADE
                               WHERE TRUNC(CA.DTHORAAGEND) BETWEEN TRUNC(SYSDATE-30) AND TRUNC(SYSDATE+30)
                              AND CA.IDSITUACAOAGEND = '4'
                              AND CA.CDOPERADOR IN ({$operadores})
                           GROUP BY AT.NMATIVIDADE, AT.NRATIVIDADE, AT.DSCORIDATIVIDADE) QUANTIDADE_CONCLUIDAS,

                          (SELECT AT.NMATIVIDADE, AT.NRATIVIDADE, COUNT(CA.NRCHECKLISTAGEND) AS QUANTIDADE_TOTAL
                             FROM FCM_CHECKLISTAGEND CA
                             JOIN FCM_ATIVIDADECHK AC
                               ON AC.NRCHECKLIST = CA.NRCHECKLIST
                             JOIN FCM_ATIVIDADE AT
                               ON AT.NRATIVIDADE = AC.NRATIVIDADE
                               WHERE TRUNC(CA.DTHORAAGEND) BETWEEN TRUNC(SYSDATE-30) AND TRUNC(SYSDATE+30)
                              AND CA.CDOPERADOR IN ({$operadores})
                         GROUP BY  AT.NMATIVIDADE, AT.NRATIVIDADE) QUANTIDADE_TOTAL
                    WHERE QUANTIDADE_TOTAL.NRATIVIDADE = QUANTIDADE_CONCLUIDAS.NRATIVIDADE";


        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }
}
