<?php

namespace FM\Service;

use FM\Factory\Fm00004001TransferirTarefasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\LogtransfAgendService;
use FM\Service\FuncionariosHierarqService;
use FM\Service\FindService;
use FM\Service\Fm00010000ParametrizacaoGeralService;

class Fm00004001TransferirTarefasService
{

    protected $connection;
    protected $Fm00004001TransferirTarefasFactory;
    protected $Environment;
    protected $newCode;
    protected $LogtransfAgendService;
    protected $FuncionariosHierarqService;
    protected $FindService;
    protected $ParametrizacaoGeralService;

    public function __construct(EntityManager $entityManager, LogtransfAgendService $LogtransfAgendService, Fm00004001TransferirTarefasFactory $Fm00004001TransferirTarefasFactory, Environment $Environment, NewCode $newCode, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, Fm00010000ParametrizacaoGeralService $ParametrizacaoGeralService)
    {
        $this->connection                           = $entityManager->getConnection();
        $this->Fm00004001TransferirTarefasFactory   = $Fm00004001TransferirTarefasFactory;
        $this->LogtransfAgendService                = $LogtransfAgendService;
        $this->Environment                          = $Environment;
        $this->newCode                              = $newCode;
        $this->FuncionariosHierarqService           = $FuncionariosHierarqService;
        $this->FindService                          = $FindService;
        $this->ParametrizacaoGeralService           = $ParametrizacaoGeralService;
    }

    public function getOperadorDestino($parametros)
    {

        $listaLocais = (isset($parametros['NRLOCALCLIENTE'])) ? $parametros['NRLOCALCLIENTE'] : $this->FuncionariosHierarqService->getLocaisHierarquia($this->Environment->getCdOperador());
        $filtro = (isset($parametros['CDOPERADOR'])) ? " AND OPERADOR.CDOPERADOR <> " . $parametros['CDOPERADOR'] : null;

        return $this->Fm00004001TransferirTarefasFactory->getOperadorDestino(array(
            'NRHABILIDADE'  => $parametros['NRHABILIDADE'],
        ), $listaLocais, $filtro);
    }

    public function transferirTarefas($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $parametrizacoes = $this->ParametrizacaoGeralService->getParametrizacoes();
            $datasConflitantes = array();

            if ($parametrizacoes[0]['IDVALIDACRIATAREFA'] === 'N') {
                foreach ($parametros['TAREFAS'] as $key => $tarefa) {
                    $this->realizarTransferencia($tarefa, $parametros['OPERADORDESTINO']);
                }
            } elseif ($parametrizacoes[0]['IDVALIDACRIATAREFA'] === 'S') {
                if ($parametrizacoes[0]['IDVALIDACONFLITOURG'] === 'N') {
                    foreach ($parametros['TAREFAS'] as $key => $tarefa) {
                        if ($tarefa['IDPRIORIDADE'] === 1) {
                            $this->realizarTransferencia($tarefa, $parametros['OPERADORDESTINO']);
                            unset($parametros['TAREFAS'][$key]);
                        }
                    }
                }

                if (count($parametros['TAREFAS']) > 0) {
                    $tarefasConflitos = array();
                
                    if ($parametros['OPERADORDESTINO'] === null) {
                        $tarefasConflitos = $this->montarDadosPosConflito($parametros['TAREFAS']);
                        
                        $this->verificarConflitoFront($tarefasConflitos);

                        foreach ($tarefasConflitos as $key => $tarefa) {
                            if ($tarefa['TAREFABANCO'] === 'N') {
                                $this->realizarTransferencia($tarefa);
                                unset($tarefasConflitos[$key]);
                            }
                        }
                    }

                    $datasConflitantes = $this->validarConflitos(empty($tarefasConflitos) ? $parametros['TAREFAS'] : $tarefasConflitos, $parametros['OPERADORDESTINO']);

                    foreach ($datasConflitantes as $key => $tarefa) {
                        if (!isset($tarefa['IGNORARTAREFA']) || $tarefa['IGNORARTAREFA'] === 'N') {
                            $this->realizarTransferencia($tarefa);
                            unset($datasConflitantes[$key]);
                        }
                    }
                }
            }
    
            $this->connection->commit();
            if (count($datasConflitantes) > 0) {
                $retorno = $this->formatarRetorno($datasConflitantes);
            } else {
                $retorno = 'OK';
            }
            return $retorno;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function formatarRetorno($parametros)
    {
        foreach ($parametros as $key => $tarefa) {
            $dataInicial = \DateTime::createFromFormat('d/m/Y H:i', $tarefa['DTHORAAGEND']);
            $dataFinal = \DateTime::createFromFormat('d/m/Y H:i', $tarefa['DTHRFINALAGEND']);
    
            $duracao = $dataInicial->diff($dataFinal);

            $parametros[$key]['DTHORAAGEND'] = $dataInicial->format('d/m/Y');
            $parametros[$key]['DTHRFINALAGEND'] = $dataFinal->format('d/m/Y');
            $parametros[$key]['HORAINICIAL'] = $dataInicial->format('H:i');
            $parametros[$key]['DURACAO'] = $duracao->format('%H:%I');
        }

        return $parametros;
    }

    public function montarDadosPosConflito($parametros)
    {
        foreach ($parametros as $key => $dados) {
            if ($dados['IGNORARTAREFA'] !== 'S') {
                $datas = $this->calcularDuracao($dados['DTHORAAGEND'], $dados['HORAINICIAL'], $dados['DURACAO']);
                $dados['DTHORAAGEND'] = $datas[0];
                $dados['DTHRFINALAGEND'] = $datas[1];
                $parametros[$key] = $dados;
            } else {
                unset($parametros[$key]);
            }
            unset($parametros[$key]['HORAINICIAL'], $parametros[$key]['DURACAO']);
        }
        return $parametros;
    }

    public function verificarConflitoFront($tarefasConflitos)
    {
        // Função para validar se as tarefas vindas do conflito estão conflitando entre si.
        $aux = array();
        foreach ($tarefasConflitos as $tarefaA) {
            foreach ($tarefasConflitos as $tarefaB) {
                if ($tarefaA['CDOPERADOR'] === $tarefaB['CDOPERADOR']) {
                    if ($tarefaA['NRCHECKLISTAGEND'] !== $tarefaB['NRCHECKLISTAGEND']) {
                        $conflito = $this->verificarConflitoDatas($tarefaA, $tarefaB);
                        if ($conflito) {
                            throw new \Exception("Operação bloqueada!<br><br> Existem conflitos de horário nas tarefas que está tentando inserir. <br><br> A Tarefa <b>". $tarefaA['NMCHECKLISTAGEND']."</b> conflitou com a tarefa <b>".$tarefaB['NMCHECKLISTAGEND']."</b>.");
                        }
                    }
                }
            }
        }
    }

    public function calcularDuracao($dataInicial, $horaInicial, $duracao)
    {
        // converter a duração vinda do front para o formato esperado pelo método DateInterval
        [$hora, $minuto] = explode(':', $duracao);
        $totalMinutos = ($hora * 60) + $minuto;

        $horarioInicial = $dataInicial . " " . $horaInicial;
        $horarioInicial =  \DateTime::createFromFormat('d/m/Y H:i', $horarioInicial);

        $horarioFinal = clone ($horarioInicial);
        $horarioFinal->add(new \DateInterval('PT' . $totalMinutos . 'M'));

        return [$horarioInicial->format('d/m/Y H:i'), $horarioFinal->format('d/m/Y H:i')];
    }

    public function realizarTransferencia($parametros, $operador = null)
    {
        $operador = ($operador !== null) ? $operador : $parametros['CDOPERADOR'];
        $cdOperador = $this->Environment->getCdOperador();
    
        $this->Fm00004001TransferirTarefasFactory->transferirTarefa(array(
                "CDOPERADOR"        => $operador,
                "CDOPERULTATU"      => $cdOperador,
                "NRORGULTATU"       => $this->Environment->getNrOrg(),
                "DTHORAAGEND"       => $parametros['DTHORAAGEND'],
                "DTHRFINALAGEND"    => $parametros['DTHRFINALAGEND'],
                "NRCHECKLISTAGEND"  => $parametros['NRCHECKLISTAGEND'],
            ));
        
        $this->inserirLog($parametros, $operador);
    }

    public function inserirLog($parametros, $operador)
    {
        $dadosHistorico = array();
        $cdOperador = $this->Environment->getCdOperador();

        array_push($dadosHistorico, $parametros);
        $dadosHistorico[0]['OPERADORORIGEM']         = $parametros['CDOPERADOR'];
        $dadosHistorico[0]['OPERADORDESTINO']        = ($operador !== null) ? $operador : $parametros['CDOPERADOR'];
        $dadosHistorico[0] ['operadorTransfer']      = $cdOperador;
        $this->LogtransfAgendService->insertLogTrasnfer($dadosHistorico[0]);
    }

    public function validarConflitos($parametros, $operador = null)
    {
        $operadorAux = null;

        foreach ($parametros as $key => $dadosFront) {
            unset($dadosFront['RNUM']);
            unset($dadosFront['TOTAL_LINES']);
        
            $operador = ($operador !== null) ? $operador : $dadosFront['CDOPERADOR'];
            
            if ($operador !== $operadorAux) {
                $dadosBanco = $this->Fm00004001TransferirTarefasFactory->getTarefasPorOperador($operador);
                $operadorAux = $operador;
            }

            if (count($dadosBanco) > 0) {
                foreach ($dadosBanco as $dadoBack) {
                    if ($operador === $dadoBack['CDOPERADOR']) {
                        $conflito = $this->verificarConflitoDatas($dadosFront, $dadoBack);

                        if ($conflito) {
                            $dadoBack['IGNORARTAREFA'] = 'S';
                            $dadoBack['TAREFABANCO'] = 'S';
                            
                            $dadosFront['CDOPERADOR'] = $operador;
                            $dadosFront['IGNORARTAREFA'] = 'S';
                            $dadosFront['TAREFABANCO'] = 'N';
                            $parametros[$key] = $dadosFront;
                            array_push($parametros, $dadoBack);
                        }
                    }
                }
            } else {
                $dadosFront['IGNORARTAREFA'] = 'N';
                $dadosFront['TAREFABANCO'] = 'N';
                $parametros[$key] = $dadosFront;
            }
        }
        return $parametros;
    }

    public function verificarConflitoDatas($dadosX, $dadosY)
    {
        // Converter as strings de data/hora para objetos DateTime
        $inicio1 = \DateTime::createFromFormat('d/m/Y H:i', $dadosX['DTHORAAGEND']);
        $fim1 = \DateTime::createFromFormat('d/m/Y H:i', $dadosX['DTHRFINALAGEND']);

        $inicio2 = \DateTime::createFromFormat('d/m/Y H:i', $dadosY['DTHORAAGEND']);
        $fim2 = \DateTime::createFromFormat('d/m/Y H:i', $dadosY['DTHRFINALAGEND']);

        // Verificar se há conflito de datas
        if (($inicio1 >= $inicio2 && $inicio1 <= $fim2) ||
            ($fim1 >= $inicio2 && $fim1 <= $fim2) ||
            ($inicio1 <= $inicio2 && $fim1 >= $fim2)
        ) {
            return true;
        }
        return false;
    }
}
