import { Utilities } from '@/util/Utilities';
import { DialogService, LoadingService, TextInput } from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00040001CriteriosAuditoriaController {
    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRCRITERIOAUDIT: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMCRITERIOAUDIT_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

     /* eslint-disable */
    public canEditRow({ row }: any) {
        return row['__added_row'] === true;
    }
    /* eslint-enable */

    public salvarCriterioAuditoria({ component, event }: any) {
        event.preventDefault();
        const criterios = component.getEditedRows();

        if (criterios.length > 0) {
            if (criterios[0].NMCRITERIOAUDIT != null) {
                LoadingService.show();
                Http.post('/salvarCriteriosAvaliacao', component.getEditedRows()).then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
            } else {
                Utilities.alertMessage('Preencha o campo de "Nome"', 'error');
            }
        } else {
            Utilities.alertMessage('Preencha o campo de "Nome"!', 'error');
        }
    }

    public excluirCriterios({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040001CriteriosAuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-criterios-avaliacao');
        LoadingService.show();
        Http.post('/excluirRegistros', {
            CRITERIOS: component.selectedRows.map((element: any) => element.NRCRITERIOAUDIT),
        }).then((response: IDictionary) => {
            if (response.data !== 0) {
                Utilities.showDialog('Operação Bloqueada', 'warning', 'Alguns critérios não puderam ser excluídos pois estão sendo utilizados em algum Modelo de Auditoria.');
            }
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
