import { Utilities } from '@/util/Utilities';
import { DialogService, LoadingService, TextInput } from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00040003CategoriaAuditoriaController {
    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRCATEGORIAAUDIT: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMCATEGORIAAUDIT_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    /* eslint-disable */
    public canEditRow({ row }: any) {
        return row['__added_row'] === true;
    }
    /* eslint-enable */

    public salvarCategoriaAuditoria({ component, event }: any) {
        event.preventDefault();
        const categoria = component.getEditedRows();

        if (categoria.length > 0) {
            if (categoria[0].NMCATEGORIAAUDIT != null) {
                LoadingService.show();
                Http.post('/salvarCategoriaAuditoria', categoria[0]).then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
            } else {
                Utilities.alertMessage('Preencha o campo de "Nome"', 'error');
            }
        } else {
            Utilities.alertMessage('Preencha o campo de "Nome"!', 'error');
        }
    }

    public excluirCategoria({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040003CategoriaAuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-categoria-auditoria');
        LoadingService.show();
        Http.post('/excluirRegistros', {
            CATEGORIAS: component.selectedRows.map((element: any) => element.NRCATEGORIAAUDIT),
        }).then((response: IDictionary) => {
            if (response.data !== 0) {
                Utilities.showDialog('Operação Bloqueada', 'warning', 'Algumas categorias não puderam ser excluídas pois estão sendo utilizadas em algum Modelo de Auditoria.');
            }
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
